#------------------ start of rules/stdRules ---------------------
#
# $Header: /cvs/stx/stx/rules/stdRules,v 1.439 2017-02-21 17:10:56 cg Exp $
#
# MIMEType: application/x-make
#
# Notice: on systems where make does not support the include
# directive, this file is included as a whole into the generated
# Makefile, and the following comment applies:
#   these rules come from $(TOP)/rules/stdRules - do not change
#   anything below, the changes will be lost when Makefiles are
#   recreated.
# If the make command does support include, this file is
# referenced to by the generated Makefile with an include directive.
#
# Do not blame me about the complexity here - nowadays, I would use
# configure. But I am happy to have a setup which runs on all those wierd
# machines and have other things to do ...
#
# these rules enable the Makefiles to:
#
#     make mf                   -- recreate the makefile after a definition change
#
#     make debug                -- compile without -O / -g instead
#
#     make objs                 -- compile to all .o-files
#
#     make obj                  -- make a prelinked obj-library file
#
#     make clobber              -- remove all that can be reconstructed
#
#     make clean                -- remove all .o-files, but leave libraries and executables
#
#     make cleanjunk            -- clean junk (intermediate) files
#
#     make notes                -- for stc-developpers only
#                                  output notes about internal actions
#
#     make warnings             -- for stc-developpers only
#                                  output warning messages
#
# ---- internal rules
#
#     make newobj               -- make a prelinked obj-library file
#                                  including a classlibrary init-function
#
#     make sharedobj            -- make a prelinked shared obj-library file
#                                  including a classlibrary init-function


# controls if a 'lib' subdirectory is to be made
# when linking against shared objects
NO_LINK_LIBDIR="yes"


# redefinableAll: Makefile $(ALL_TARGET) postMake
redefinableAll: $(ALL_TARGET) postMake

postMake::

#
# notice: the rule below used to be the Makefile rule;
# however, since gmake automatically executes this rule upon entry,
# this may lead to trouble, if you are on an NFS mounted file system,
# and the machines times are not completely in sync (i.e. withing a second).
# To avoid endless make recursion, this rule is now called 'mf'
#

MAKEFILE=makefile
#MAKEFILE=makefile.gen
RELEASE=`cat $(TOP)/RELEASE`

#
# debug - make with -g instead of -O
#
debug:
	$(MAKE) OPT="-g $(OPT)" LIBRUN_OPT="-g $(LIBRUN_OPT)" STCOPT="$(STCOPT)"

#
# notes:
#    output notes on st-code (for compiler writer only)
#
notes::
	@-if [ "*.st" != '\*.st' ]; then                 \
	  for i in ""*.st""; do                          \
	    echo notes on $$i ...;                       \
	    $(STC) -check +notice +hint $(STCFLAGS) $(STC_MSGFORMAT) $$i; \
	  done;                                          \
	fi
	@-if [ "$(SUBDIRS)"x != x ]; then  \
	  for i in $(SUBDIRS)" "; do       \
	    if [ -d $$i ]; then            \
	      echo "notes in $$i";         \
	      (cd $$i && $(MAKE) notes );   \
	    fi;                            \
	  done;                            \
	fi

#
# warnings:
#    output warning on st-code (for compiler writer only)
#
warnings::
	@-if [ "*.st" != '\*.st' ]; then                 \
	  for i in ""*.st""; do                          \
	    echo warnings on $$i ...;                       \
	    $(STC) -check +hint $(STCFLAGS) $(STC_MSGFORMAT) $$i; \
	  done;                                          \
	fi
	@-if [ "$(SUBDIRS)"x != x ]; then  \
	  for i in $(SUBDIRS)" "; do       \
	    if [ -d $$i ]; then            \
	      echo "warnings in $$i";         \
	      (cd $$i && $(MAKE) warnings );   \
	    fi;                            \
	  done;                            \
	fi

#
# hints:
#    output hints on st-code (mostly for compiler writer)
#
hints::
	@-for i in *.st; do                           \
	    echo hints on $$i ...;                    \
	    $(STC) -check +hint $(STCFLAGS) $(STC_MSGFORMAT) $$i;      \
	done

#
# fullcheck:
#    outputs notice + hints
#
fullcheck::
	@-if [ "*.st" != '\*.st' ]; then                 \
	  for i in *.st; do                              \
	    echo checking $$i ...;                       \
	    $(STC) -check $(STCFLAGS) $(STC_MSGFORMAT) +notice +hint $$i; \
	  done;                                          \
	fi
	@-if [ "$(SUBDIRS)"x != x ]; then    \
	  for i in $(SUBDIRS)" "; do         \
	    if [ -d $$i ]; then              \
	      echo "check in $$i";           \
	      (cd $$i && $(MAKE) fullcheck ); \
	    fi;                              \
	  done;                              \
	fi

UNCRITICAL:
	$(MAKE) $(SPACEOPT_O_RULE) STFILE=$(NAME) STCOPT="$(STCOPT) $(UNCRITICALOPT)" CLASSLIB_CC="$(CLASSLIB_CC)" CLASSLIB_OPT="$(CLASSLIB_OPT)" CC="$(CLASSLIB_CC)" OPT="$(OPT)"

$(UNCRITICALOBJS):
	@$(MAKE) UNCRITICAL CC="$(CLASSLIB_CC)" OPT="$(OPT)" CLASSLIB_CC="$(CLASSLIB_CC)" CLASSLIB_OPT="$(CLASSLIB_OPT)" NAME=$*

AUTOLOADED:
	@-if [ -f i_$(NAME).o ]; then \
	    echo "assuming stubObject i_$(NAME).o is up-to-date"; \
	else \
	    $(MAKE) $(SPACEOPT_O_RULE) STFILE=$(NAME) STCOPT="$(STCOPT)" FORCEAUTOLOAD="+autoload -sepInitCode" CLASSLIB_CC="$(CLASSLIB_CC)" CLASSLIB_OPT="$(CLASSLIB_OPT)" OPT="$(OPT)" CC="$(CLASSLIB_CC)"; \
	fi

$(AUTOLOADEDOBJS):
	@$(MAKE) AUTOLOADED CLASSLIB_CC="$(CLASSLIB_CC)" CLASSLIB_OPT="$(CLASSLIB_OPT)" CC="$(CC)" OPT="$(OPT)" NAME=$*

#
# objs:
#    generate all .o-files
#

subDirObjsIfBeforeObjs:
	@-if [ "$(SUBDIRS_AFTER_OBJS)" = "" ]; then \
	    $(MAKE) subDirObjsIfPresent; \
	fi

objs:: pre_objs pre_local_objs $(OBJS) post_local_objs

pre_objs::

pre_local_objs::

post_local_objs::

clsFiles:: pre_cls subDirClassFiles
	@test  "$(CLS)" = "" || $(MAKE) cls_files

cls_files:: $(CLS)

#
# The makefile in SUBDIR may not have been built yet...
#
subDirClassFiles::
	@-if [ "$(SUBDIRS)" != "" ]; then \
	    $(MAKE) subDirClassFilesPresent;    \
	fi

subDirClassFilesPresent::
	@for i in $(SUBDIRS); do \
	    if [ -d "$$i" ]; then \
		echo "making cls-files in $$i ..."; \
		(cd $$i && $(MAKE) makefile && $(MAKE) clsFiles) || exit $? ;  \
	    fi;  \
	done

pre_cls::

# systems which support hiding symbols from libs,
# may add the following lines in the config-file:
#
# PRE_LD_ACTION=createHiddenSymFile
# POST_LD_ACTION=removeHiddenSymFile


emptyRule:
	@true

createHiddenSymFile: hiddenSyms.stc
	@-echo > /dev/null

removeHiddenSymFile:
	@-rm -f hiddenSyms.stc


#
# make without optimizer ...
#
nonopt:
	$(MAKE) OPT=


#
# those two are no longer needed
#
CLASSLIST_OR_EMPTY=

#
# rule to build a class Library
#
lib: classLibRule

classLibRule: clsFiles $(LOCAL_EXTRA_TARGETS) $(OBJTARGET) registerClassLib

#
# create a symbolic link to the shared library in the projects/smalltalk/packages
# folder. This folder can later be scanned quickly and presented in the Launcher's
# load package menu item.
#
REGISTERED_PACKAGES_DIR=$(TOP)/projects/smalltalk/packages

registerClassLib: $(REGISTERED_PACKAGES_DIR)/$(LIBNAME)$(SONAME)

$(REGISTERED_PACKAGES_DIR)/$(LIBNAME)$(SONAME): | $(REGISTERED_PACKAGES_DIR)
	@echo "registering $(MODULE):$(MODULE_DIR)/$(LIBNAME) in $(REGISTERED_PACKAGES_DIR)"
	rm -f $(REGISTERED_PACKAGES_DIR)/$(LIBNAME)$(SONAME)
	echo "lib ../../../../$(MODULE)/$(MODULE_DIR)/$(LIBNAME)$(SONAME)" > $(REGISTERED_PACKAGES_DIR)/$(LIBNAME)$(SONAME)

$(REGISTERED_PACKAGES_DIR):
	@mkdir $(REGISTERED_PACKAGES_DIR)

registerApplication:
	@-if [ "$(APPNAME)"x != ""x ]; then \
	    echo "registering $(MODULE):$(MODULE_DIR)/$(APPNAME) in $(REGISTERED_PACKAGES_DIR)";  \
	    rm -f $(REGISTERED_PACKAGES_DIR)/$(APPNAME); \
	    echo "app ../../../../$(MODULE)/$(MODULE_PATH)/$(APPNAME)" > $(REGISTERED_PACKAGES_DIR)/$(APPNAME); \
	fi

#
# rule to build a program (application)
#
progRule:
	@-if [ "$(PROGS)"x != ""x ]; then \
	  for i in $(PROGS)" "; do        \
	    echo "making $$i";            \
	    $(MAKE) PROGTARGET$(TARGET_RULE_EXTENSION) PROG=$$i; \
	  done;                           \
	fi

#
# one of the two below is performed by the $(OBJTARGET) target;
# create a library for either static or dynamic linking
# (this is controlled by the TARGET_EXTENSION, set in an architecture specific config file)
#

obj::    obj$(TARGET_RULE_EXTENSION)

obj_shared: newobj $(LIBNAME)$(SONAME)


obj_shared_stub:
	@-if [ -f $(STUBLIBNAME)$(OBJNAME) ]; then \
	  echo "making shared stub object $(STUBLIBNAME)$(SONAME)"; \
	  $(MAKE) $(STUBLIBNAME)$(SONAME); \
	else \
	  echo " ** $(STUBLIBNAME)$(SONAME) not made - no $(STUBLIBNAME)$(OBJNAME)"; \
	fi

obj_static: newobj

#
# newobj:
#    generate all .o-files, pre-link them into one big lib.o file
#    create a lib-init function and include it in library
#

newobj_prelink::
	@true "nothing done here"

newobj_postlink::
	@true "nothing done here"

newobj: objs $(PRE_LD_ACTION) newobj_prelink newobj_link newobj_postlink $(POST_LD_ACTION)

newobj_link:: $(LIBNAME)$(OBJNAME)

$(LIBNAME)$(OBJNAME): $(LIBNAME)Init.o $(LINKOBJS)
	@echo "linking class object ..."
	$(CLASSLIB_LD) $(LD_REL_FLAG) $(LD_OBJ_FLAGS) $^ $(LD_OBJ_LIBS) $(LD_REL_OBJ_LIBS) -o $@

$(LIBNAME).$(SO): $(LIBNAME)$(OBJNAME)
	@echo "linking class library ..."
	$(LD) -shared $(LDFLAGS) -o $@ $< $(LOCAL_SHARED_LIBS) $(CLASSLIB_LIBS)
	$(OBJCOPY) --only-keep-debug $@ $@.debug
	$(CHMOD) ugo-x $@.debug
	$(STRIP) --strip-debug --strip-unneeded $@
	$(OBJCOPY) --add-gnu-debuglink="$@.debug" $@

newobj_linkSep::
	@-echo "linking sepInit class object ..."
	@-echo $(CLASSLIB_LD) $(LD_REL_FLAG) $(LD_OBJ_FLAGS) \
	    $(LIBNAME)Init.o `$(TOP)/rules/i_list $(LINKOBJS)` $(LINKOBJS) $(LD_OBJ_LIBS) $(LD_REL_OBJ_LIBS) -o $(LIBNAME)$(OBJNAME)
	@$(CLASSLIB_LD) $(LD_REL_FLAG) $(LD_OBJ_FLAGS) \
	    $(LIBNAME)Init.o `$(TOP)/rules/i_list $(LINKOBJS)` $(LINKOBJS) $(LD_OBJ_LIBS) $(LD_REL_OBJ_LIBS) -o $(LIBNAME)$(OBJNAME)

newobj_link_sepInitCode::
	@-if [ "$(SEPINITCODE)" != "" ]; then   \
	    $(MAKE) newobj_linkSep LINKOBJS="$(LINKOBJS)" LIBNAME=$(LIBNAME);                     \
	else                                    \
	    echo "** oops - sepInit rule but no +sepInit specified (non fatal)"; \
	    $(MAKE) newobj_link LINKOBJS="$(LINKOBJS)" LIBNAME=$(LIBNAME);                        \
	fi

stub_link::
	@-echo "linking stub class object ..."
	@-echo $(CLASSLIB_LD) $(LD_REL_FLAG) $(LD_OBJ_FLAGS) $(STUBLIBNAME)Stub.o $(LINKOBJS) -o $(STUBLIBNAME)$(OBJNAME)
	@$(CLASSLIB_LD) $(LD_REL_FLAG) $(LD_OBJ_FLAGS) $(STUBLIBNAME)Stub.o $(LINKOBJS) -o $(STUBLIBNAME)$(OBJNAME)

iobjs:
	echo "making sepInit objs ..."
	@-if [ "$(OBJS)"x != x ]; then \
	  for i in $(OBJS); do         \
	      $(MAKE) i_$$i ;          \
	  done;                        \
	fi

#
# to force creation of a shared object, even if not defined in config
#
sharedobj: newobj
	@echo "making shared object $(LIBNAME)$(SONAME)"
	$(MAKE) $(LIBNAME)$(SONAME)

#
# objstrip:
#     like make obj, but strip debug syms
#
objstrip::
	@if [ "$(OBJ_SUBDIRS)"x != x ]; then               \
	  for i in $(OBJ_SUBDIRS)" "; do                    \
	    if [ -d $$i ]; then                         \
	      echo "making obj in $$i";                 \
	      (cd $$i && $(MAKE) obj );                  \
	    fi;                                         \
	  done;                                         \
	fi
	@if [ "$(LIBNAME)" != "" ]; then                \
	  $(MAKE) objs &&                               \
	  echo $(CLASSLIB_LD) $(LD_REL_FLAG) $(LD_STRIP_FLAG) $(LD_OBJ_FLAGS) $(LINKOBJS) -o $(LIBNAME)$(OBJNAME) && \
	  rm -f $(LIBNAME)$(OBJNAME) &&                 \
	  $(CLASSLIB_LD) $(LD_REL_FLAG) $(LD_STRIP_FLAG) $(LD_OBJ_FLAGS) $(LINKOBJS) -o $(LIBNAME)$(OBJNAME) && \
	  chmod -x,+w $(LIBNAME)$(OBJNAME);                \
	fi

# cleanjunk:
#    remove all temporary stuff
#
cleanjunk::

clean::
	@echo cleaning
	@-rm -f err out *.bak core .dir.*.old *.old *~ *.H ttt* i_[A-Z]*.[cso]
	@-rm -f .dependencies
	@-if [ "$(LIBNAME)"x != x ]; then \
	    rm -f $(LIBNAME)Init.[co] $(STUBLIBNAME)Stub.[co]; \
	    if [ -f $(LIBNAME).so ]; then \
		    rm -f $(LIBNAME).o; \
	    fi; \
	fi
	@-rm -f bitmaps/*.bak bitmaps/*~
	@-rm -f resources/*.bak resources/*~ resources/*.old
	@-if [ "$(ALLSUBDIRS)"x != x ]; then \
	  $(MAKE) cleanALLSUBDIRS; \
	fi
	@-if [ "$(CLEANSUBDIRS)"x != x ]; then \
	  $(MAKE) cleanCLEANSUBDIRS; \
	fi

cleanALLSUBDIRS::
	@-for i in $(ALLSUBDIRS); do \
	    if [ -d $$i ]; then \
	      if [ -f $$i/Make.proto ]; then \
		  echo "cleaning $$i ..."; \
		  (cd $$i && $(MAKE) clean); \
	      fi; \
	    fi; \
	done

cleanCLEANSUBDIRS::
	@-for i in $(CLEANSUBDIRS); do \
	    if [ -d $$i ]; then \
	      if [ -f $$i/Make.proto ]; then \
		  echo "cleaning $$i ..."; \
		  (cd $$i && $(MAKE) clean); \
	      fi; \
	    fi; \
	done

#
# clobber:
#    remove all that can be regenerated
#
CLOBBERED_OBJS=*.o

clobber::
	@-rm -f err out *.bak core .dir.*.old *.old *~ $(CLOBBERED_OBJS) \
	       *.H *.so *.obj obj \
	       liblist.stc \
	       .listOfSTfiles* .dependencies
	@rm -f *.a $(PROGS) $(TESTPROGS)
	@-if [ "$(LIBNAME)"x != x ]; then \
	    rm -f $(LIBNAME)Init.[co] $(STUBLIBNAME)Stub.[co] $(LIBNAME)zip; \
	fi
	@-rm -f bitmaps/*.bak bitmaps/*~
	@-rm -f resources/*.bak resources/*~ resources/*.old
	@-if [ "$(ALLSUBDIRS)"x != x ]; then \
	  $(MAKE) clobberALLSUBDIRS;         \
	fi
	@-if [ "$(CLEANSUBDIRS)"x != x ]; then \
	  $(MAKE) clobberCLEANSUBDIRS;         \
	fi

clobberALLSUBDIRS::
	@-for i in $(ALLSUBDIRS); do      \
	    if [ -d $$i ]; then           \
	      echo "clobbering $$i ...";  \
	      (cd $$i && $(MAKE) clobber); \
	    fi;                           \
	done

clobberCLEANSUBDIRS::
	@-for i in $(CLEANSUBDIRS); do    \
	    if [ -d $$i ]; then           \
	      echo "clobbering $$i ...";  \
	      (cd $$i && $(MAKE) clobber); \
	    fi;                           \
	done

#
# distClean:
#    cleanup for distribution
#
distClean:: clobber
	@-if [ "$(ALLSUBDIRS)"x != x ]; then \
	  $(MAKE) distCleanALLSUBDIRS;       \
	fi
	@-if [ "$(CLEANSUBDIRS)"x != x ]; then \
	  $(MAKE) distCleanCLEANSUBDIRS;       \
	fi

distCleanALLSUBDIRS::
	@-for i in $(ALLSUBDIRS); do        \
	    if [ -d $$i ]; then             \
	      if [ -f $$i/$(MAKEFILE) ]; then  \
		echo "distClean  $$i ...";  \
		(cd $$i && $(MAKE) distClean); \
	      fi;                           \
	    fi;                             \
	done

distCleanCLEANSUBDIRS::
	@-for i in $(CLEANSUBDIRS); do      \
	    if [ -d $$i ]; then             \
	      if [ -f $$i/$(MAKEFILE) ]; then  \
		echo "distClean $$i ...";   \
		(cd $$i && $(MAKE) distClean); \
	      fi;                           \
	    fi;                             \
	done
#
# copy all relevant files to $(DELIVERDIR)/full
#
preDeliverFull::

postDeliverFull::

deliverFull::
	@if [ "$(RELDIR)" = "" ]; then  \
	    echo "no RELDIR defined";   \
	    exit 1;                     \
	fi
	@if [ "$(DELIVERDIR)" = "" ]; then      \
	    echo "no DELIVERDIR defined";       \
	    exit 1;                             \
	fi
	-$(MAKE) preDeliverFull RELDIR=$(RELDIR) DELIVERDIR=$(DELIVERDIR)
	-$(MAKE) deliverFullLocal RELDIR=$(RELDIR) DELIVERDIR=$(DELIVERDIR)
	@-if [ "$(ALLSUBDIRS)"x != x ]; then            \
	  $(MAKE) deliverFullALLSUBDIRS RELDIR=$(RELDIR) DELIVERDIR=$(DELIVERDIR);              \
	fi
	@-if [ "$(DELIVERSUBDIRS)"x != x ]; then        \
	  $(MAKE) deliverFullDELIVERSUBDIRS RELDIR=$(RELDIR) DELIVERDIR=$(DELIVERDIR);          \
	fi
	$(MAKE) postDeliverFull RELDIR=$(RELDIR) DELIVERDIR=$(DELIVERDIR)

deliverFullLocal::
	@if [ "$(RELDIR)" = "" ]; then  \
	    echo "no RELDIR defined";   \
	    exit 1;                     \
	fi
	@if [ "$(DELIVERDIR)" = "" ]; then \
	    echo "no DELIVERDIR defined";  \
	    exit 1;                        \
	fi
	@test -d $(DELIVERDIR) || mkdir $(DELIVERDIR)
	@test -d $(DELIVERDIR)/$(RELDIR) || mkdir $(DELIVERDIR)/$(RELDIR)
	@-if [ "$(DELIVERSOURCES)" != "" ]; then \
	    (tar cf - $(DELIVERSOURCES) | (cd $(DELIVERDIR)/$(RELDIR) && tar xvf -)); \
	fi
	@-if [ "$(DELIVERMISC)" != "" ]; then \
	    (tar cf - $(DELIVERMISC) | (cd $(DELIVERDIR)/$(RELDIR) && tar xvf -)); \
	fi
	@-if [ "$(DELIVERBINARIES)" != "" ]; then \
	    (tar cf - $(DELIVERBINARIES) | (cd $(DELIVERDIR)/$(RELDIR) && tar xvf -)); \
	fi

deliverFullALLSUBDIRS::
	@-for i in $(ALLSUBDIRS); do                    \
	    if [ -d $$i ]; then                         \
		echo "deliver files in $$i ...";        \
		(cd $$i && $(MAKE) deliverFull RELDIR=$(RELDIR)/$$i DELIVERDIR=$(DELIVERDIR));           \
	    fi;                                         \
	done

deliverFullDELIVERSUBDIRS::
	@-for i in $(DELIVERSUBDIRS); do                \
	    if [ -d $$i ]; then                         \
		echo "deliver files in $$i ...";        \
		(cd $$i && $(MAKE) deliverFull RELDIR=$(RELDIR)/$$i DELIVERDIR=$(DELIVERDIR));           \
	    fi;                                         \
	done

pre_all::

#
# all:
#    make what is appropriate here (library or application)
#
all::   pre_all
	@-if [ "$(SUBDIRS)"x != ""x ]; then \
	  $(MAKE) allSubDirs;               \
	fi

# full:
#    in addition to all, this also builds support stuff needed
#    by some subsystems using ant and possibly other tools.
#    for now, this is libjava only
full::   all

allSubDirs:
	@-for i in $(SUBDIRS); do     \
	  if [ -d $$i ]; then         \
	    $(MAKE) subDirMake SUBDIR=$$i; \
	  else                        \
	    echo "*** oops where is directory $$i ?? in "`pwd`" (non fatal)"; \
	  fi;                         \
	done

subDirMake:
	@-if [ -f $(SUBDIR)/$(MAKEFILE) ] ; then \
	    true; \
	else \
	    $(MAKE) subMakefile SUBDIR=$(SUBDIR); \
	fi
	@-echo "********************************"
	@-echo "making all in $(SUBDIR)"
	@-echo "********************************"
	@-cd $(SUBDIR) && $(MAKE) all

allLibs::
	@-if [ "$(LIBNAME)"x != ""x ]; then \
	  $(MAKE) lib;               \
	fi
	@-if [ "$(SUBDIRS)"x != ""x ]; then \
	  $(MAKE) allSubDirLibs;               \
	fi

allSubDirLibs:
	@-for i in $(SUBDIRS); do     \
	  if [ -d $$i ]; then         \
	    $(MAKE) subDirLibsMake SUBDIR=$$i; \
	  else                        \
	    echo "*** oops where is directory $$i ?? in "`pwd`" (non fatal)"; \
	  fi;                         \
	done

subDirLibsMake:
	@-if [ -f $(SUBDIR)/$(MAKEFILE) ] ; then \
	    true; \
	else \
	    $(MAKE) subMakefile SUBDIR=$(SUBDIR); \
	fi
	@-echo "********************************"
	@-echo "making libs in $(SUBDIR)"
	@-echo "********************************"
	@-cd $(SUBDIR) && $(MAKE) allLibs

APPLICATION:
	    @-rm -f main.o modulList.stc modulList.c modulList.o
	    $(MAKE) main.o \
		STARTUP_CLASS="$(STARTUP_CLASS)" \
		STARTUP_SELECTOR="$(STARTUP_SELECTOR)" \
		MAIN_DEFINES="$(MAIN_DEFINES)"
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    $(MAKE) linkTarget \
		LIBLIST="$(LIBLIST)" \
		LIBOBJS="$(LIBOBJS)" \
		LINK_LIBOBJS="$(LINK_LIBOBJS)" \
		TARGET=$(TARGET) \
		ADDITIONAL_CLASSES="$(APPLICATION_CLASSES)" \
		ADDITIONAL_OBJS="$(APPLICATION_OBJS)" \
		ADDITIONAL_LINK_OBJS="$(APPLICATION_LINK_OBJS)"
	    @-rm -f main.o modulList.stc modulList.c modulList.o

#
# a standAlone target without graphical user interface
#
NON_GUI_APPLICATION:
	    @-rm -f main.o modulList.stc modulList.c modulList.o
	    $(MAKE) main.o \
		STARTUP_CLASS="$(STARTUP_CLASS)" \
		STARTUP_SELECTOR="$(STARTUP_SELECTOR)" \
		MAIN_DEFINES="$(MAIN_DEFINES)"
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    $(MAKE) linkTarget \
		LIBLIST="$(APPLICATION_LIBLIST)" \
		LIBOBJS="$(APPLICATION_LIBOBJS)" \
		LINK_LIBOBJS="$(APPLICATION_LINK_LIBOBJS)" \
		TARGET=$(TARGET) \
		ADDITIONAL_CLASSES="$(APPLICATION_CLASSES)" \
		ADDITIONAL_OBJS="$(APPLICATION_OBJS)" \
		ADDITIONAL_LINK_OBJS="$(APPLICATION_LINK_OBJS)"
	    @-rm -f main.o modulList.stc modulList.c modulList.o

xNON_GUI_APPLICATION:
	    @-rm -f main.o modulList.stc modulList.c modulList.o
	    $(MAKE) main.o \
		STARTUP_CLASS="$(STARTUP_CLASS)" \
		STARTUP_SELECTOR="$(STARTUP_SELECTOR)" \
		MAIN_DEFINES="$(MAIN_DEFINES)"
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    $(MAKE) linkTarget \
		LIBLIST="$(MINI_LIST) libstx_libbasic2 libstx_libboss $(APPLICATION_LIBLIST)" \
		LIBOBJS="$(MINI_LIB) $(LIBBASIC2) $(LIBBOSS) $(APPLICATION_LIBOBJS)" \
		LINK_LIBOBJS="$(LINK_MINI_LIB) $(LINK_LIBBASIC2) $(LINK_LIBBOSS) $(APPLICATION_LINK_LIBOBJS)" \
		TARGET=$(TARGET) \
		ADDITIONAL_CLASSES="$(APPLICATION_CLASSES)" \
		ADDITIONAL_OBJS="$(APPLICATION_OBJS)" \
		ADDITIONAL_LINK_OBJS="$(APPLICATION_LINK_OBJS)"
	    @-rm -f main.o modulList.stc modulList.c modulList.o

#
# a minimum standAlone target without graphical user interface
# (only links libbasic - no compiler etc.)
#
BARE_NON_GUI_APPLICATION:
	    @-rm -f main.o modulList.stc modulList.c modulList.o
	    $(MAKE) main.o \
		STARTUP_CLASS="$(STARTUP_CLASS)" \
		STARTUP_SELECTOR="$(STARTUP_SELECTOR)" \
		MAIN_DEFINES="$(MAIN_DEFINES)"
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    $(MAKE) linkTarget \
		LIBLIST="$(BARE_LIST) $(APPLICATION_LIBLIST)" \
		LIBOBJS="$(BARE_LIB) $(APPLICATION_LIBOBJS)" \
		LINK_LIBOBJS="$(LINK_BARE_LIB) $(APPLICATION_LINK_LIBOBJS)" \
		TARGET=$(TARGET) \
		ADDITIONAL_CLASSES="$(APPLICATION_CLASSES)" \
		ADDITIONAL_OBJS="$(APPLICATION_OBJS)" \
		ADDITIONAL_LINK_OBJS="$(APPLICATION_LINK_OBJS)"
	    @-rm -f main.o modulList.stc modulList.c modulList.o

#
# with graphical user interface
#
BARE_GUI_APPLICATION:
	@-rm -f main.o modulList.stc modulList.c modulList.o
	$(MAKE) main.o \
	    STARTUP_CLASS="$(STARTUP_CLASS)" \
	    STARTUP_SELECTOR="$(STARTUP_SELECTOR)" \
	    MAIN_DEFINES="$(MAIN_DEFINES)"
	@-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	$(MAKE) linkTarget \
	    LIBLIST="$(EXTRA_UI_CLASSES) $(APPLICATION_LIBLIST)" \
	    LIBOBJS="$(EXTRA_UI_OBJ) $(APPLICATION_LIBOBJS)" \
	    LINK_LIBOBJS="$(EXTRA_LINK_UI_OBJ) $(APPLICATION_LINK_LIBOBJS)" \
	    TARGET=$(TARGET) \
	    ADDITIONAL_CLASSES="$(APPLICATION_CLASSES)" \
	    ADDITIONAL_OBJS="$(APPLICATION_OBJS)" \
	    ADDITIONAL_LINK_OBJS="$(APPLICATION_LINK_OBJS)"
	@-rm -f main.o modulList.stc modulList.c modulList.o

#
# with tool libraries
#
GUI_APPLICATION:
	@echo making GUI_APPLICATION...
	@echo EXTRA_UI_CLASSES=$(EXTRA_UI_CLASSES)
	@echo EXTRA_UI_OBJ=$(EXTRA_UI_OBJ)
	@echo EXTRA_LINK_UI_OBJ=$(EXTRA_LINK_UI_OBJ)
	@-rm -f main.o modulList.stc modulList.c modulList.o
	$(MAKE) main.o \
	    STARTUP_CLASS="$(STARTUP_CLASS)" \
	    STARTUP_SELECTOR="$(STARTUP_SELECTOR)" \
	    MAIN_DEFINES="$(MAIN_DEFINES)"
	@-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	$(MAKE) linkTarget \
	    LIBLIST="$(EXTRA_UI_CLASSES) $(APPLICATION_LIBLIST)" \
	    LIBOBJS="$(EXTRA_UI_OBJ) $(APPLICATION_LIBOBJS)" \
	    LINK_LIBOBJS="$(EXTRA_LINK_UI_OBJ) $(APPLICATION_LINK_LIBOBJS)" \
	    TARGET=$(TARGET) \
	    ADDITIONAL_CLASSES="$(APPLICATION_CLASSES)" \
	    ADDITIONAL_OBJS="$(APPLICATION_OBJS)" \
	    ADDITIONAL_LINK_OBJS="$(APPLICATION_LINK_OBJS)"
	@-rm -f main.o modulList.stc modulList.c modulList.o
	@$(MAKE) registerClassLib

#
# a program target
# needs: CLASSLIBRARIES,
#        STARTUP_SELECTOR, STARTUP_CLASS
#        LIBOBJS
#
PROGTARGET_shared::
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    @-echo "Making PROGTARGET_shared ..."
	    $(MAKE) linkTarget                      \
		    LIBOBJS="$(LIBOBJS_SHARED)" \
		    ADDITIONAL_OBJS="$(ADDITIONAL_OBJS)"    \
		    ADDITIONAL_CLASSES="$(ADDITIONAL_CLASSES)"    \
		    ADDITIONAL_LIBS="$(ADDITIONAL_LIBS)"    \
		    EXTRA_LIBS="$(EXTRA_LIBS)" \
		    LINK_LIBRUN="$(LINK_LIBRUN)" \
		    LIBRUN=$(LIBRUN)

PROGTARGET_static:
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    @-echo "Making PROGTARGET_static ..."
	    $(MAKE) linkTarget                      \
		    LIBOBJS="$(STATIC_LIBOBJS)" \
		    LINK_LIBOBJS="$(STATIC_LIBOBJS)" \
		    ADDITIONAL_OBJS="$(STATIC_ADDITIONAL_OBJS)"    \
		    ADDITIONAL_CLASSES="$(ADDITIONAL_CLASSES)"    \
		    ADDITIONAL_LIBS="$(ADDITIONAL_LIBS)"    \
		    EXTRA_LIBS="$(STATIC_EXTRA_LIBS)" \
		    EXTRA_UI_OBJ="$(STATIC_EXTRA_UI_OBJ)" \
		    LINK_LIBRUN="$(LIBRUN)" \
		    LIBRUN="$(LIBRUN)"

PROGTARGET_fullstatic:
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    @-echo "Making PROGTARGET_fullstatic ..."
	    $(MAKE) linkTarget                      \
		    LIBOBJS="$(STATIC_LIBOBJS)" \
		    LINK_LIBOBJS="$(STATIC_LIBOBJS)" \
		    ADDITIONAL_OBJS="$(STATIC_ADDITIONAL_OBJS)"    \
		    ADDITIONAL_CLASSES="$(ADDITIONAL_CLASSES)"    \
		    ADDITIONAL_LIBS="$(ADDITIONAL_LIBS)"    \
		    LINK_LIBRUN="$(LIBRUN)" \
		    LIBRUN="$(LIBRUN)" \
		    MATHLIB="$(STATIC_MATHLIB)" \
		    EXTRA_LIBS="$(STATIC_EXTRA_LIBS)" \
		    EXTRA_UI_OBJ="$(STATIC_EXTRA_UI_OBJ)" \
		    SYSLIBS="$(STATIC_SYSLIBS)"

PROGTARGET::
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    @-echo "Making PROGTARGET_shared ..."
	    $(MAKE) linkTarget                      \
		    LIBOBJS="$(LIBOBJS_SHARED)" \
		    ADDITIONAL_OBJS="$(ADDITIONAL_OBJS)"    \
		    ADDITIONAL_CLASSES="$(ADDITIONAL_CLASSES)"    \
		    ADDITIONAL_LIBS="$(ADDITIONAL_LIBS)"    \
		    EXTRA_LIBS="$(EXTRA_LIBS)" \
		    LINK_LIBRUN="$(LINK_LIBRUN)" \
		    LIBRUN=$(LIBRUN)

nullRebaseRule:
	    @-echo "No rebase done for this architecture/configuration"

linkTarget: main.$(O) $(LIBOBJS) checkSymbolsInLibrunAreUpToDate $(LIBRUN) $(EXTRA_OBJ)
	    @-echo "Making modulList (PACK = " $(CONF_PACKS) ") ..."
	    @-echo "LIBLIST=" $(LIBLIST)
	    @-echo "ADDITIONAL_CLASSES=" $(ADDITIONAL_CLASSES)
	    $(MAKE) modulList.$(O) \
		    LIBS="$(LIBOBJS)" \
		    LIBLIST="$(LIBLIST)" \
		    ADDITIONAL_CLASSES="$(ADDITIONAL_CLASSES)"
	    @-echo "OPTIONAL_SHARED_LIBS=" $(OPTIONAL_SHARED_LIBS)
	    @-echo "linking ..."
	    $(MAKE) linkLibs LINK_LIBDIR="$(LINK_LIBDIR)" LINK_LIBOBJS="$(LINK_LIBOBJS)" LIBOBJS="$(LIBOBJS)"
	    $(MAKE) $(rebaseLibsRule) LINK_LIBRUN="$(LINK_LIBRUN)" LINK_LIBOBJS="$(LINK_LIBOBJS)" LINK_ADDITIONAL_OBJS="$(LINK_ADDITIONAL_OBJS)"
	    $(LD) $(ST_LDFLAG) $(LDFLAGS) -o $(TARGET)$(EXE) \
		  $(CRT0) modulList.$(O) main.$(O) \
		  $(OPTIONAL_SHARED_LIBS) \
		  $(LINK_LIBOBJS) $(LINK_ADDITIONAL_OBJS) \
		  $(LINK_LIBRUN) \
		  $(ADDITIONAL_LIBS) $(MATHLIB) $(EXTRA_LIBS) $(SYSLIBS) $(CRTN)
	    @if [ "$(POST_LD)" != "" ]; then \
		echo $(POST_LD) $(TARGET)$(EXE); \
		$(POST_LD) $(TARGET)$(EXE); \
	    fi
	    @if [ "$(POST_LD2)" != "" ]; then \
		echo $(POST_LD2) $(TARGET)$(EXE); \
		$(POST_LD2) $(TARGET)$(EXE); \
	    fi

# a pure C library
cLibRule:
	@-echo "linking ..."
	$(LD) $(ST_LDFLAG) $(LDFLAGS) -shared -o $(CLIB)$(SONAME) \
	      $(CRT0) \
	      $(OPTIONAL_SHARED_LIB_PATH) \
	      $(COBJS) $(LINK_ADDITIONAL_OBJS) \
	      $(ADDITIONAL_LIBS) $(MATHLIB) $(EXTRA_LIBS) $(SYSLIBS) $(CRTN)

checkSymbolsInLibrunAreUpToDate:
	if [ -f $(LIBRUNDIR)/symbolAddress.h.timestamp ]; then \
	    $(MAKE) reallyCheckSymbolsInLibrunAreUpToDate; \
	fi

reallyCheckSymbolsInLibrunAreUpToDate: $(LIBRUNDIR)/symbolAddress.h.timestamp

$(LIBRUNDIR)/symbolAddress.h.timestamp: $(INCLUDE)/symbols.stc $(LIBRUNDIR)/symbols.h
	@-echo "Making librun for changed symbolList..."
	cd $(LIBRUNDIR); $(MAKE)

#
# install:
#     make install
#
install::

.PHONY:	install

#
# rule to make an archive
#
$(LIB):     pre_lib $(DEPOBJS) $(DEPAUXOBJS) $(LIB_HELPER)
	-cp $(LIB) $(LIB).sav
	rm -f $(LIB)
	$(AR) $(AR_LIB_FLAG) $(LIB) $(AR_OBJS)
	-$(RANLIB) $(LIB)

pre_lib::

CONF_STRING="$(CONF):$(CONF_PACKS)"

#
# rule to make a main (either in smalltalk or application)
# main.o depends on Makefile, to be recompiled when the configuration
# changes (used to be in libbasic/Smalltalk.st)
#
# Now always rebuild main.o, to get the newest build number/date

#main.o::  $(MAIN) $(INCLUDE)/stc.h $(MAKEFILE)
main.o::
	  $(MAIN_CC) -o main.o -c $(MAIN_CFLAGS) $(MAIN_DEFINES) \
		-DSTARTUP_CLASS="\"$(STARTUP_CLASS)\"" \
		-DSTARTUP_SELECTOR="\"$(STARTUP_SELECTOR)\"" \
		-DCONFIGURATION_STRING="\"$(CONF_STRING)\"" \
		-DBUILD_DATE="\"`date '+%Y-%m-%d %R %Z'`\"" \
		-DBUILD_NUMBER="\"${BUILD_NUMBER}\"" \
		-I$(INCLUDE) -I. $(MAIN)

#
# liblist.stc:
#    automatic generate a liblist.stc - libinfo file from all OBJ files
#
liblist.stc:: Make.proto Make.spec
	@-if [ "*.st" != '\*.st' ]; then                \
	  if [ "$(LIBNAME)" != "" ]; then               \
	    echo "";                                    \
	    echo "generating libinfo liblist.stc ...";  \
	    echo "";                                    \
	    if [ -f liblist.stc ]; then                 \
		rm -f liblist.stc;                      \
	    fi;                                         \
	    for i in ""*.st"" ; do                      \
	      NAME=`basename $$i .st`;                  \
	      if [ -f "$$NAME".o ]; then                \
	       CLASS=`$(STC) -extractClass $$i`;        \
	       if [ "$$CLASS" != "" ]; then             \
		 echo "  adding "$$i;                   \
		 echo $$CLASS" "$(LIBNAME) >> liblist.stc;       \
	       fi;                                      \
	      fi;                                       \
	    done;                                       \
	    $(MAKE) patchLiblist;                       \
	  fi;                                           \
	fi


forceAbbrev::
	@-$(MAKE) saveAbbrev
	@-$(MAKE) abbrev.stc

saveAbbrev:
	@-if [ -f abbrev.stc ]; then \
	  rm -f abbrev.stc.bak ;     \
	  mv abbrev.stc abbrev.stc.bak ; \
	fi

#
# abbrev.stc:
#    generate an abbrev.stc - abbreviation file
#    from all .st files found here; leave as is, if no change
#
abbrev.stc:: Make.proto Make.spec $(TOP)/rules/rightstr .listOfSTfiles
	@-if [ "`echo *.st`" != "*.st" ]; then \
	  echo "generating abbrev.stc in "$(MODULE_DIR)"..."; \
	  LC_COLLATE=POSIX; export LC_COLLATE; \
	  $(MAKE) genAbbrev; \
	  if [ "$(TOUCH)" != "echo" ]; then   \
	    $(TOUCH) abbrev.stc; \
	  fi; \
	  if [ -f abbrev.stc.bak ]; then \
	    if (cmp abbrev.stc.bak abbrev.stc >/dev/null); then  \
	      echo "  abbrev.stc unchanged";  \
	      mv abbrev.stc.bak abbrev.stc;   \
	    else \
	      echo "  abbrev has changed";    \
	    fi \
	  fi \
	else \
	  echo "  no st-files -> no abbrevs"; \
	fi

$(TOP)/rules/rightstr:
	@-if [ "$(CONF)" = "NT/cygwin32-bc" ]; then \
	    (cd $(TOP)/rules && $(CC) -c rightstr.c); \
	    (cd $(TOP)/rules && $(CC) rightstr.$(O)); \
	else \
	    (cd $(TOP)/rules && $(CC) -o rightstr rightstr.c); \
	fi




.listOfSTfiles: .
	@-if [ -r .listOfSTfiles ]; then                        \
	    ls *.st > .listOfSTfiles.new;                       \
	    if cmp .listOfSTfiles .listOfSTfiles.new; then      \
		rm -f .listOfSTfiles.new;                       \
	    else                                                \
		rm -f .listOfSTfiles;                           \
		mv .listOfSTfiles.new .listOfSTfiles;           \
	    fi;                                                 \
	else                                                    \
	    ls *.st > .listOfSTfiles;                           \
	fi


modulList.o: modulList.c
	$(CC) -c $(CFLAGS) -I$(INCLUDE) modulList.c
	@-echo "done with modulList.o"

modulList.c: modulList.stc
	@$(MAKE) $(CLASSLISTEDITRULE) CLASSLIST=modulList
	@-echo "done with modulList.c"

modulList.stc: $(MAKEFILE)
	@-rm -f modulList.stc modulList.c
	@$(MAKE) GENERICLIST CLASSLIST=modulList                \
		 ADDITIONAL_CLASSES="$(ADDITIONAL_CLASSES)"     \
		 LIBLIST="$(LIBLIST)"                           \
		 CLASSLIBRARIES="                               \
			$(LIBLIST)                              \
			$(OTHERLIBRARIES)                       \
		 "                                              \
		 CLASSLISTS="                                   \
			$(EXTRACLASSLISTS)                      \
			$(CLIENTCLASSLISTS)                     \
		 "

#
# edit a modulList.c file from a modulList.stc file
#
classListEdit:
	@echo "making $(CLASSLIST).c from $(CLASSLIST).stc"
	@echo "#include <stcIntern.h>" > $(CLASSLIST).c
	@echo "typedef void (*vf)();" >> $(CLASSLIST).c
	@sed '/^$$/d' < $(CLASSLIST).stc > $(CLASSLIST).tmp1
	@sed 's/ $$//' < $(CLASSLIST).tmp1 > $(CLASSLIST).tmp2
	@sed 's/ $$//' < $(CLASSLIST).tmp2 > $(CLASSLIST).tmp1
	@sed 's/^/extern void _/' < $(CLASSLIST).tmp1 > $(CLASSLIST).tmp2
	@sed -e '/[A-Za-z]/s/$$/_Init();/' -e 's/::/__/' < $(CLASSLIST).tmp2 >> $(CLASSLIST).c
	@echo "static vf modules[] = {" >> $(CLASSLIST).c
	@sed 's/^/_/' < $(CLASSLIST).tmp1 > $(CLASSLIST).tmp2
	@sed 's/::/__/' < $(CLASSLIST).tmp2 > $(CLASSLIST).tmp1
	@sed 's/$$/_Init,/' < $(CLASSLIST).tmp1 >> $(CLASSLIST).c
	@echo "(vf)0" >> $(CLASSLIST).c
	@echo "};" >> $(CLASSLIST).c
	@echo "vf *__modules__ = modules;" >> $(CLASSLIST).c
	@-rm -f $(CLASSLIST).tmp1 $(CLASSLIST).tmp2
	@-echo "done with classListEdit"

#
# generate a classList.stc/modulList.stc
# from LIBLIST, CLASSLISTS and EXTRA_CLASSES
# requires CLASSLIST to be set
#
GENERICLIST:   $(CLASSLISTS)
	    @echo "making $(CLASSLIST)"
	    @-rm -f $(CLASSLIST)
	    @-if [ "$(LIBLIST)" != "" ]; then                   \
		echo "   included class libraries:";            \
		$(MAKE) addNamesToList NAMES="$(LIBLIST)"; \
	     fi
	    @-if [ "$(CLASSLISTS)" = "" ]; then                 \
	      echo "(ignore any following syntax error ...)" >/dev/null;        \
	     fi
	    @-if [ "$(CLASSLISTS)" != "" ]; then                \
		echo "   included individual classes:";         \
		for i in "$(CLASSLISTS)"; do                    \
		    echo "     $$i";                            \
		done;                                           \
		cat $(CLASSLISTS) >> $(CLASSLIST).stc;          \
	     fi
	    @-if [ "$(ADDITIONAL_CLASSES)" != "" ]; then        \
		echo "   included additional (user) classes:";  \
		for i in $(ADDITIONAL_CLASSES); do              \
		    echo "     $$i" | sed 's/_/_137/g';         \
		    echo $$i | sed 's/_/_137/g'   >> $(CLASSLIST).stc;  \
		done; \
	    fi
	    @echo

addNamesToList:
	@-for i in $(NAMES); do \
	    echo "     $$i";    \
	    echo $$i    >> $(CLASSLIST).stc; \
	done

#
# generate a libXXXInit.c file from classList/modulList.stc
#
$(LIBNAME)Init.o: libInit.cc
	$(CLASSLIB_CC) $(CLASSLIB_CFLAGS) -x c -c $< -o $@

#
# create symbolic links for all used libraries in the LINK_LIBDIR
# directory. Req'd for sharedLib links.
#
mkLinkLibDir:
	@test -d $(LINK_LIBDIR) || mkdir $(LINK_LIBDIR)

linkLibsDir:
	@-if [ "$(LINK_LIBDIR)" = "" ]; then                            \
	    if [ "$(NO_LINK_LIBDIR)" != "yes" ]; then                   \
		test -d lib || mkdir lib;                               \
	    fi                                                          \
	else                                                            \
	    $(MAKE) mkLinkLibDir LINK_LIBDIR="$(LINK_LIBDIR)";          \
	fi

linkLibs:
	@if [ "$(LINK_LIBOBJS)" != "$(LIBOBJS)" ]; then                \
	    if [ "$(LIBOBJS)" != "" ]; then                             \
		$(MAKE) linkLibsDir LINK_LIBDIR="$(LINK_LIBDIR)";       \
		$(MAKE) linkObjs LIBOBJS="$(LIBOBJS)";                  \
	    fi;                                                         \
	fi
	@if [ "$(LINK_LIBRUN)" != "$(LIBRUN)" ]; then                  \
	    $(MAKE) linkLibsDir LINK_LIBDIR="$(LINK_LIBDIR)";           \
	    if [ "$(LINK_LIBDIR)" = "" ]; then                          \
		if [ "$(LIBRUN)" = "" ]; then                           \
		  echo "  symlinking to $(LIBRUNDIR)/$(LINK_LIBRUN) for linkage ..."; \
		  rm -f `basename $(LINK_LIBRUN)` lib/`basename $(LINK_LIBRUN)`;      \
		  $(LNS) $(LIBRUNDIR)/$(LINK_LIBRUN) .;                 \
		  if [ "$(NO_LINK_LIBDIR)" != "yes" ]; then             \
		    (cd lib && $(LNS) ../$(LIBRUNDIR)/$(LINK_LIBRUN) .); \
		  fi;                                                   \
		else                                                    \
		  echo "  symlinking to $(LIBRUN) for linkage ...";     \
		  rm -f `basename $(LIBRUN)` lib/`basename $(LIBRUN)`;  \
		  $(LNS) $(LIBRUN) .;                                   \
		  if [ "$(NO_LINK_LIBDIR)" != "yes" ]; then             \
		      (cd lib && $(LNS) ../$(LIBRUN) .);                 \
		  fi;                                                   \
		fi;                                                     \
	    else                                                        \
		if [ "$(FORCE_LINK_LIBRUN)" != "" ]; then               \
		    echo "  symlinking to $(LIBRUNDIR)/$(LINK_LIBRUN) for linkage ..."; \
		    rm -f `basename $(LINK_LIBRUN)` lib/`basename $(LINK_LIBRUN)`;      \
		    $(LNS) $(LIBRUNDIR)/$(LINK_LIBRUN) .;               \
		    if [ "$(NO_LINK_LIBDIR)" != "yes" ]; then           \
			(cd lib && $(LNS) ../$(LIBRUNDIR)/$(LINK_LIBRUN) .); \
		    fi;                                                 \
		else                                                    \
		    if [ "$(LINK_LIBDIR)" != "" ]; then                 \
			echo "  symlinking to ../$(LIBRUN) for linkage ..."; \
			rm -f $(LINK_LIBDIR)/`basename $(LIBRUN)`;      \
			(cd $(LINK_LIBDIR) && $(LNS) ../$(LIBRUN) .);    \
		    fi;                                                 \
		fi;                                                     \
	    fi;                                                         \
	fi
	@if [ "$(LINK_EXTRA_OBJ)" != "$(EXTRA_OBJ)" ]; then            \
	    $(MAKE) linkLibsDir LINK_LIBDIR="$(LINK_LIBDIR)";           \
	    $(MAKE) linkExtraObj EXTRA_OBJ="$(EXTRA_OBJ)";              \
	fi

linkObjs:
	@for i in $(LIBOBJS); do                                       \
	    echo "  symlinking to $$i for linkage ...";                 \
	    if [ "$(LINK_LIBDIR)" = "" ]; then                          \
		rm -f `basename $$i` lib/`basename $$i`;                \
		$(LNS) $$i .;                                           \
		if [ "$(NO_LINK_LIBDIR)" != "yes" ]; then               \
		    (cd lib && $(LNS) ../$$i .);                         \
		fi                                                      \
	    else                                                        \
		if [ "$(LINK_LIBDIR)" != "" ]; then                     \
		    rm -f $(LINK_LIBDIR)/`basename $$i`;                \
		    (cd $(LINK_LIBDIR) && $(LNS) ../$$i .);              \
		fi;                                                     \
	    fi;                                                         \
	done

linkExtraObj:
	@for i in $(EXTRA_OBJ); do                                     \
	    echo "  symlinking to $$i for linkage ...";                 \
	    if [ "$(LINK_LIBDIR)" = "" ]; then                          \
		rm -f `basename $$i` lib/`basename $$i`;                \
		$(LNS) $$i .;                                           \
		if [ "$(NO_LINK_LIBDIR)" != "yes" ]; then               \
		    (cd lib && $(LNS) ../$$i .);                         \
		fi                                                      \
	    else                                                        \
		if [ "$(LINK_LIBDIR)" != "" ]; then                     \
		    rm -f $(LINK_LIBDIR)/`basename $$i`;                \
		    (cd $(LINK_LIBDIR) && $(LNS) ../$$i .);              \
		fi;                                                     \
	    fi;                                                         \
	done

removeLinkLibs::
	@-if [ "$(LINK_LIBDIR)" = "" ]; then                            \
	    rm -f $(EXTRA_LINK_LIBS) $(EXTRA_LINK_OBJ)                  \
		  $(LINK_LIBOBJS) $(LINK_LIBRUN);                       \
	fi

%.o: %.st
	$(STC) $(STCFLAGS) $(STC_MSGFORMAT) $(CFLAGS_ARCH) $(CLASSLIB_CFLAGS) $(CFLAGS_LOCAL) -CC="$(CLASSLIB_CC)" -c $(PWD)/$<  && \
	    $(TOUCH) $(PWD)/$@

%.c: %.st
	$(STC) $(STCFLAGS) $(STC_MSGFORMAT) $(CFLAGS_ARCH) $(CCLASSLIB_FLAGS) $(CFLAGS_LOCAL) -CC="$(CLASSLIB_CC)" -C $(PWD)/$<


#
# Following is for compatibility with existing Make.proto's
#
includeLink: